"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const execa = require("execa");
const fs = require("fs");
const rimraf = require("rimraf");
const path = require("path");
const mkdirp = require("mkdirp");
const util_1 = require("./util");
function installDependencies(projectDir, project, opts) {
    const args = ["install", "--ignore-scripts"];
    if (opts.force) {
        args.push("--force");
    }
    execa.sync("yarn", args, { cwd: projectDir });
    util_1.logger.iterateInfo(`installed dependencies for ${project.package}`, opts.countOf);
}
exports.installDependencies = installDependencies;
function buildProject(projectDir, project, opts) {
    if (project.lerna) {
        execa.sync("lerna", ["run", "build"], { cwd: projectDir });
        util_1.logger.iterateInfo(`built ${project.package}`, opts.countOf);
        return;
    }
    const buildCommand = project.buildCommand || "build";
    execa.sync("yarn", [buildCommand], { cwd: projectDir });
    util_1.logger.iterateInfo(`built ${project.package}`, opts.countOf);
    return;
}
exports.buildProject = buildProject;
function linkSelf(projectDir, project, opts) {
    let cwd = projectDir;
    if (project.linkFrom) {
        cwd = path.join(projectDir, project.linkFrom);
    }
    if (!project.lerna) {
        execa.sync("yarn", ["unlink"], { cwd });
        execa.sync("yarn", ["link"], { cwd });
    }
    else {
        execa.sync("lerna", ["exec", "yarn", "unlink"], { cwd });
        execa.sync("lerna", ["exec", "yarn", "link"], { cwd });
    }
    util_1.logger.iterateInfo(`linked ${project.package}`, opts.countOf);
    return;
}
exports.linkSelf = linkSelf;
function linkDependencies(projectDir, project, opts) {
    if (project.links) {
        if (project.lerna) {
            execa.sync("lerna", ["exec", "--", "yarn", "link", ...project.links], { cwd: projectDir });
        }
        else {
            execa.sync("yarn", ["link", ...project.links], { cwd: projectDir });
        }
        util_1.logger.iterateInfo(`linked dependencies for ${project.package}`, opts.countOf);
        return;
    }
}
exports.linkDependencies = linkDependencies;
function cloneTypeDefinitions(baseDir, typeDefs) {
    for (let [typeDefName, typeInfo] of Object.entries(typeDefs)) {
        cloneTypeDefinition(baseDir, typeDefName, typeInfo);
    }
}
exports.cloneTypeDefinitions = cloneTypeDefinitions;
function cloneTypeDefinition(baseDir, typeDefName, typeInfo) {
    const typeDefPath = path.join(baseDir, "@types", typeDefName);
    if (!fs.existsSync(typeDefPath)) {
        mkdirp.sync(typeDefPath);
    }
    if (util_1.repoExists(typeDefPath)) {
        util_1.logger.warn(`Type repository for ${typeDefName} is present`);
        checkoutBranch(typeDefPath, typeInfo.branch);
        return;
    }
    util_1.logger.warn("setting up typeDefs, this could take a while...");
    execa.sync("git", [
        "clone",
        typeInfo.remote,
        typeDefPath,
        "--branch",
        typeInfo.branch,
        "--depth",
        typeInfo.depth ? typeInfo.depth.toString() : "1"
    ], {
        cwd: typeDefPath
    });
    return util_1.logger.info(`cloned typeDefinition for ${typeDefName}`);
}
exports.cloneTypeDefinition = cloneTypeDefinition;
function linkType(type, projectDir, baseDir, typeDef) {
    const typeSymbolicPath = path.join(projectDir, `node_modules/@types/${type}`);
    const typePath = path.join(baseDir, "@types", typeDef.typesPath);
    rimraf.sync(typeSymbolicPath);
    fs.symlinkSync(typePath, typeSymbolicPath, "dir");
}
exports.linkType = linkType;
function linkTypes(projectDir, project, config) {
    const { baseDir, typeDefs } = config;
    if (project.types) {
        for (let type of project.types) {
            linkType(type, projectDir, baseDir, typeDefs[type]);
        }
    }
}
exports.linkTypes = linkTypes;
function cloneProject(remote, projectDir) {
    if (!util_1.repoExists(projectDir)) {
        execa.sync("git", ["clone", remote, projectDir]);
    }
}
exports.cloneProject = cloneProject;
function checkoutBranch(projectDir, branch) {
    if (!util_1.repoExists(projectDir)) {
        throw Error(`Cannot checkout ${branch}, ${projectDir} is not a git repository`);
    }
    return execa.sync("git", ["checkout", branch], { cwd: projectDir });
}
exports.checkoutBranch = checkoutBranch;
function cloneAndInstall(projectDir, project, opts) {
    cloneProject(project.remote, projectDir);
    checkoutBranch(projectDir, project.branch);
    util_1.logger.iterateInfo(`cloned and checked out ${project.package}#${project.branch}`, opts.countOf);
    installDependencies(projectDir, project, opts);
}
exports.cloneAndInstall = cloneAndInstall;
